!
!    Copyright  2013, 2014, 2020, 2021 Guy Munhoven
!
!    This file is part of SolveSAPHE v. 2

!    SolveSAPHE is free software: you can redistribute it and/or modify
!    it under the terms of the GNU Lesser General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SolveSAPHE is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU Lesser General Public License for more details.
!
!    You should have received a copy of the GNU Lesser General Public License
!    along with SolveSAPHE.  If not, see <http://www.gnu.org/licenses/>.
!





! **********************
! Precompiler directives
! **********************

! CREATEFILES:
! -- if defined, result files are written
! -- if not defined, result files are *not* written

#define CREATEFILES


! CASE_TSP = 1: Surface cold
!            2: Surface warm
!            3: Deep cold
!            4: Surface cold, brackish

#define CASE_TSP 1


! CASE_SW  = 1: SW1 -- common present-day seawater
!            2: SW2 -- common present-day and future seawater
!            3: SW3 -- extreme seawater samples
!            4: SW4 -- brackish water and dilute seawater

#define CASE_SW 2


! CASE_DIC = 1: DIC
!            2: CO2
!            3: HCO3
!            4: CO3

#define CASE_DIC 1


! CASE_ZHINI:
! -- if not defined, leave the initialisation to the solver
!    (solver default)
! -- if defined, use explicit initialisation (pH=8 for all)

#undef CASE_ZHINI


! **************************
! End precompiler directives
! **************************


!===============================================================================
PROGRAM DRIVER_AT_GENERAL2
!===============================================================================

USE MOD_PRECISION
USE MOD_CHEMCONST
USE MOD_PHSOLVERS


IMPLICIT NONE

REAL(KIND=wp)                       :: z_alktot, z_dicvar, z_bortot
REAL(KIND=wp)                       :: z_po4tot, z_siltot, z_nh4tot, z_h2stot
REAL(KIND=wp)                       :: z_so4tot, z_flutot
REAL(KIND=wp), DIMENSION(2)         :: z_hini, z_h, z_val
REAL(KIND=wp), DIMENSION(2)         :: z_hini2

REAL                                :: z_cputime_start, z_cputime_end

INTEGER                             :: ji, ji_alk, ji_dic
INTEGER                             :: n_roots

INTEGER, PARAMETER                  :: jp_resunit = 1



! Temperature, salinity, pressure data
! ====================================

#if CASE_TSP == 1
REAL(KIND=wp), PARAMETER            :: z_t_k    = 275.15_wp
REAL(KIND=wp), PARAMETER            :: z_s      =  35.00_wp
REAL(KIND=wp), PARAMETER            :: z_p_bar  =   0.00_wp
 CHARACTER(LEN=*), PARAMETER        :: cp_tspid = 'sc'
#endif

#if CASE_TSP == 2
REAL(KIND=wp), PARAMETER            :: z_t_k    = 298.15_wp
REAL(KIND=wp), PARAMETER            :: z_s      =  35.00_wp
REAL(KIND=wp), PARAMETER            :: z_p_bar  =   0.00_wp
 CHARACTER(LEN=*), PARAMETER        :: cp_tspid = 'sw'
#endif

#if CASE_TSP == 3
REAL(KIND=wp), PARAMETER            :: z_t_k    = 275.15_wp
REAL(KIND=wp), PARAMETER            :: z_s      =  35.00_wp
REAL(KIND=wp), PARAMETER            :: z_p_bar  = 300.00_wp
 CHARACTER(LEN=*), PARAMETER        :: cp_tspid = 'dc'
#endif

#if CASE_TSP == 4
REAL(KIND=wp), PARAMETER            :: z_t_k   = 275.15_wp
REAL(KIND=wp), PARAMETER            :: z_s     =   3.50_wp
REAL(KIND=wp), PARAMETER            :: z_p_bar =   0.00_wp
 CHARACTER(LEN=*), PARAMETER        :: cp_tspid = 'sb'
#endif


! DIC selection
! =============

#if CASE_DIC == 1
 CHARACTER(LEN=*), PARAMETER        :: cp_dicid = 'ct-'
 INTEGER,          PARAMETER        :: jp_dicsel = jp_dic
#endif

#if CASE_DIC == 2
 CHARACTER(LEN=*), PARAMETER        :: cp_dicid = 'co2-'
 INTEGER,          PARAMETER        :: jp_dicsel = jp_co2
#endif

#if CASE_DIC == 3
 CHARACTER(LEN=*), PARAMETER        :: cp_dicid = 'hco3-'
 INTEGER,          PARAMETER        :: jp_dicsel = jp_hco3
#endif

#if CASE_DIC == 4
 CHARACTER(LEN=*), PARAMETER        :: cp_dicid = 'co3-'
 INTEGER,          PARAMETER        :: jp_dicsel = jp_co3
#endif



! DIC and Alkalinity distributions (SWx cases)
! ============================================

#if CASE_SW == 1
! Settings for common present-day seawater samples
# if CASE_DIC == 1
! DIC interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =   1.85E-3_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =   2.45E-3_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

# if CASE_DIC == 2
! CO2 interval extent (log scale)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  -6.00_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  -3.00_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

# if CASE_DIC == 3
! HCO3 interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =   1.20E-3_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =   2.40E-3_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

# if CASE_DIC == 4
! CO3 interval extent (log scale)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  -5.00_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  -3.00_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

! ALK interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_alktot_min =  2.20E-3_wp
REAL(KIND=wp), PARAMETER            :: z_alktot_max =  2.50E-3_wp
INTEGER, PARAMETER                  :: jp_nalk = 300

CHARACTER(LEN=*), PARAMETER        :: cp_fileid = 'sw1-' // cp_dicid // cp_tspid
#endif



#if CASE_SW == 2
! Settings for common present-day and future seawater
! (derived from SP750 simulation experiment with MBM-Medusa)
# if CASE_DIC == 1
! DIC interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  1.85E-3_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  3.35E-3_wp
INTEGER, PARAMETER                  :: jp_ndic = 1500
# endif

# if CASE_DIC == 2
! CO2 interval extent (log scale)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  -7.00E+0_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  -3.00E+0_wp
INTEGER, PARAMETER                  :: jp_ndic = 1500
# endif

# if CASE_DIC == 3
! HCO3 interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  0.60E-3_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  3.20E-3_wp
INTEGER, PARAMETER                  :: jp_ndic = 1500
# endif

# if CASE_DIC == 4
! CO3 interval extent (log scale)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  -6._wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  -3._wp
INTEGER, PARAMETER                  :: jp_ndic = 1500
# endif

! ALK interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_alktot_min =  2.2E-3_wp
REAL(KIND=wp), PARAMETER            :: z_alktot_max =  3.5E-3_wp
INTEGER, PARAMETER                  :: jp_nalk = 1300

CHARACTER(LEN=*), PARAMETER        :: cp_fileid = 'sw2-' // cp_dicid // cp_tspid
#endif



#if CASE_SW == 3
! Settings for extreme seawater samples
# if CASE_DIC == 1
! DIC interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  0.0E-3_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  6.0E-3_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

# if CASE_DIC == 2
! CO2 interval extent (log scale)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min = -14._wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  -2._wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

# if CASE_DIC == 3
! HCO3 interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  0.0E-3_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  5.0E-3_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

# if CASE_DIC == 4
! CO3 interval extent (log scale)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min = -14._wp !-9.4_wp !
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  -2._wp !-9.3_wp ! limit at -9.39_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

! ALK interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_alktot_min = -1.0E-3_wp
REAL(KIND=wp), PARAMETER            :: z_alktot_max =  5.0E-3_wp
INTEGER, PARAMETER                  :: jp_nalk = 600

CHARACTER(LEN=*), PARAMETER        :: cp_fileid = 'sw3-' // cp_dicid // cp_tspid
#endif



#if CASE_SW == 4
! Settings for dilute seawater samples
# if CASE_DIC == 1
! DIC interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  0.0E-3_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  1.2E-3_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

# if CASE_DIC == 2
! CO2 interval extent (log scale)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min = -12.00E+0_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  -3.00E+0_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

# if CASE_DIC == 3
! HCO3 interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  0.0E-3_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  1.0E-3_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

# if CASE_DIC == 4
! CO3 interval extent (log scale)
REAL(KIND=wp), PARAMETER            :: z_dicvar_min =  -9.00E+0_wp
REAL(KIND=wp), PARAMETER            :: z_dicvar_max =  -3.00E+0_wp
INTEGER, PARAMETER                  :: jp_ndic = 600
# endif

! ALK interval extent (linear)
REAL(KIND=wp), PARAMETER            :: z_alktot_min =  0.0E-3_wp
REAL(KIND=wp), PARAMETER            :: z_alktot_max =  1.5E-3_wp
INTEGER, PARAMETER                  :: jp_nalk = 600

CHARACTER(LEN=*), PARAMETER        :: cp_fileid = 'sw4-' // cp_dicid // cp_tspid
#endif


! Work variables and arrays
! =========================

REAL(KIND=wp), DIMENSION(jp_ndic)            :: z_dicvar_arr
REAL(KIND=wp), DIMENSION(jp_nalk)            :: z_alktot_arr
REAL(KIND=wp)                                :: z_dicvar_del
REAL(KIND=wp)                                :: z_alktot_del

REAL(KIND=wp), DIMENSION(jp_ndic,jp_nalk, 2) :: z_h_atgen
REAL(KIND=wp), DIMENSION(jp_ndic,jp_nalk, 2) :: z_val_atgen

INTEGER, DIMENSION(jp_ndic,jp_nalk, 2)       :: jndivg_atgen
INTEGER, DIMENSION(jp_ndic,jp_nalk, 2)       :: jniter_atgen
INTEGER, DIMENSION(jp_ndic,jp_nalk)          :: jnroots_atgen


REAL(KIND=wp), DIMENSION(jp_ndic,jp_nalk, 2) :: z_h_atsec
REAL(KIND=wp), DIMENSION(jp_ndic,jp_nalk, 2) :: z_val_atsec

INTEGER, DIMENSION(jp_ndic,jp_nalk, 2)       :: jndivg_atsec
INTEGER, DIMENSION(jp_ndic,jp_nalk, 2)       :: jniter_atsec
INTEGER, DIMENSION(jp_ndic,jp_nalk)          :: jnroots_atsec


#if defined(CASE_ZHINI_SAFEBOUNDS)
REAL(KIND=wp)                             :: zdelta, zalknw_inf, zalknw_sup
#endif



 PRINT*
 PRINT*, 'T(K)          : ', z_t_k
 PRINT*, 'Salinity      : ', z_s
 PRINT*, 'Pressure(bar) : ', z_p_bar


z_bortot =  A_BTOT_SALIN(z_s)
z_po4tot =  0.5E-6_wp
z_siltot =  5.E-6_wp
z_nh4tot =  0.E-3_wp
z_h2stot =  0.E-3_wp
z_so4tot =  A_SO4TOT_SALIN(z_s)
z_flutot =  A_FTOT_SALIN(z_s)



 PRINT*
 PRINT*, 'SigB          : ', z_bortot
 PRINT*, 'SigPO4        : ', z_po4tot
 PRINT*, 'SigSil        : ', z_siltot
 PRINT*, 'SigAmmonium   : ', z_nh4tot
 PRINT*, 'SigSulfides   : ', z_h2stot
 PRINT*, 'SigSulfates   : ', z_so4tot
 PRINT*, 'SigF          : ', z_flutot

 CALL SETUP_API4PHSWS(z_t_k, z_s, z_p_bar)

 PRINT*
 PRINT*, 'Pi_1 DIC : ', api1_dic
 PRINT*, 'Pi_2 DIC : ', api2_dic
 PRINT*, 'Pi_1 BT  : ', api1_bor
 PRINT*, 'Pi_1 PO4 : ', api1_po4
 PRINT*, 'Pi_2 PO4 : ', api2_po4
 PRINT*, 'Pi_3 PO4 : ', api3_po4
 PRINT*, 'Pi_1 Sil : ', api1_sil
 PRINT*, 'Pi_1 NH4 : ', api1_nh4
 PRINT*, 'Pi_1 H2S : ', api1_h2s
 PRINT*, 'Pi_1 HSO4: ', api1_so4
 PRINT*, 'Pi_1 HF  : ', api1_flu
 PRINT*, 'Pi_1 H2O : ', api1_wat
 PRINT*, 'pHscale : ',  aphscale


 z_dicvar_del = (z_dicvar_max-z_dicvar_min)/REAL(jp_ndic,KIND=wp)
 z_alktot_del = (z_alktot_max-z_alktot_min)/REAL(jp_nalk,KIND=wp)

 PRINT*
 PRINT*, 'Running variant "'//cp_fileid//'"'
#if defined(CASE_ZHINI)
 PRINT*, 'Initialisation procedure: explicit (pH=8)'
#else
 PRINT*, 'Initialisation procedure: default from solver (cubic polynomials)'
#endif

 PRINT*
#if CASE_DIC == 1
 PRINT*, 'DIC Interval   : ', z_dicvar_min, z_dicvar_max
 PRINT*, 'DIC Step       : ', z_dicvar_del
#endif

#if CASE_DIC == 2
 PRINT*, 'log(CO2) Interval  : ', z_dicvar_min, z_dicvar_max
 PRINT*, 'log(CO2) Step      : ', z_dicvar_del
#endif

#if CASE_DIC == 3
 PRINT*, 'HCO3 Interval  : ', z_dicvar_min, z_dicvar_max
 PRINT*, 'HCO3 Step      : ', z_dicvar_del
#endif

#if CASE_DIC == 4
 PRINT*, 'log(CO3) Interval  : ', z_dicvar_min, z_dicvar_max
 PRINT*, 'log(CO3) Step      : ', z_dicvar_del
#endif

 PRINT*, 'ALK Interval   : ', z_alktot_min, z_alktot_max
 PRINT*, 'ALK Step       : ', z_alktot_del


 DO ji_dic = 1, jp_ndic
  z_dicvar_arr(ji_dic) =  z_dicvar_min+(REAL(ji_dic,KIND=wp)-0.5_wp) * z_dicvar_del
 ENDDO

 DO ji_alk = 1, jp_nalk
  z_alktot_arr(ji_alk) =  z_alktot_min+(REAL(ji_alk,KIND=wp)-0.5_wp) * z_alktot_del
 ENDDO

#if CASE_DIC == 1
 PRINT*, 'DIC First/Last : ', z_dicvar_arr(1), z_dicvar_arr(jp_ndic)
#endif

#if CASE_DIC == 2
 PRINT*, 'CO2 First/Last : ', z_dicvar_arr(1), z_dicvar_arr(jp_ndic)
#endif

#if CASE_DIC == 3
 PRINT*, 'HCO3 First/Last: ', z_dicvar_arr(1), z_dicvar_arr(jp_ndic)
#endif

#if CASE_DIC == 4
 PRINT*, 'CO3 First/Last : ', z_dicvar_arr(1), z_dicvar_arr(jp_ndic)
#endif

 PRINT*, 'ALK First/Last : ', z_alktot_arr(1), z_alktot_arr(jp_nalk)




 PRINT*
 PRINT*, 'Calling SOLVE_AT_GENERAL2'
 PRINT*, '-------------------------'


 z_hini = 1.E-8_wp

 jndivg_atgen(:,:, :) = 0

 CALL CPU_TIME(z_cputime_start)


DO ji_alk = 1, jp_nalk

  z_alktot = z_alktot_arr(ji_alk)

  DO ji_dic = 1, jp_ndic

#if CASE_DIC == 1
    z_dicvar = z_dicvar_arr(ji_dic)
#endif

#if CASE_DIC == 2
    z_dicvar = 10._wp**(z_dicvar_arr(ji_dic))
#endif

#if CASE_DIC == 3
    z_dicvar = z_dicvar_arr(ji_dic)
#endif

#if CASE_DIC == 4
    z_dicvar = 10._wp**(z_dicvar_arr(ji_dic))
#endif

    z_h = SOLVE_AT_GENERAL2(z_alktot, z_dicvar,  z_bortot, &
                                      z_po4tot,  z_siltot, &
                                      z_nh4tot,  z_h2stot, &
                                      z_so4tot,  z_flutot, &
                                      jp_dicsel, n_roots,  &
#ifdef CASE_ZHINI
                                      p_hini=z_hini,       &
#endif
                                      p_val=z_val          )

    jnroots_atgen(ji_dic,ji_alk)      = n_roots
    jniter_atgen(ji_dic,ji_alk,:)     = niter_atgen(:)


    z_h_atgen(ji_dic,ji_alk, :)       = z_h(:)
    z_val_atgen(ji_dic,ji_alk, :)     = z_val(:)


    IF (n_roots > 0) THEN
      IF (z_h(1) == pp_hnan) &
      jndivg_atgen(ji_dic,ji_alk, 1)  = 1
    ENDIF

    IF (n_roots > 1) THEN
      IF (z_h(2) == pp_hnan) &
      jndivg_atgen(ji_dic,ji_alk, 2)  = 1
    ENDIF

  ENDDO

ENDDO


 CALL CPU_TIME(z_cputime_end)

 PRINT*
 PRINT*, '[DRIVER_AT_GENERAL2] elapsed time [s]           : ', z_cputime_end - z_cputime_start
 PRINT*, '[DRIVER_AT_GENERAL2] total number of calls      : ', jp_nalk*jp_ndic
 PRINT*, '[DRIVER_AT_GENERAL2] total number of divergences: ', SUM(jndivg_atgen)


#if defined(CREATEFILES)
 OPEN(UNIT=jp_resunit, FILE='atgen_'//cp_fileid//'.res',FORM='UNFORMATTED')

 WRITE(UNIT=jp_resunit) jp_ndic, jp_nalk
 WRITE(UNIT=jp_resunit) z_dicvar_min
 WRITE(UNIT=jp_resunit) z_dicvar_max
 WRITE(UNIT=jp_resunit) z_alktot_min
 WRITE(UNIT=jp_resunit) z_alktot_max
 WRITE(UNIT=jp_resunit) z_dicvar_arr
 WRITE(UNIT=jp_resunit) z_alktot_arr
 WRITE(UNIT=jp_resunit) z_bortot
 WRITE(UNIT=jp_resunit) z_po4tot
 WRITE(UNIT=jp_resunit) z_siltot
 WRITE(UNIT=jp_resunit) z_nh4tot
 WRITE(UNIT=jp_resunit) z_h2stot
 WRITE(UNIT=jp_resunit) z_so4tot
 WRITE(UNIT=jp_resunit) z_flutot

 WRITE(UNIT=jp_resunit) ((jnroots_atgen(ji_dic,ji_alk),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk)

 WRITE(UNIT=jp_resunit) (((z_h_atgen(ji_dic,ji_alk,ji),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk),ji=1,2)
 WRITE(UNIT=jp_resunit) (((z_val_atgen(ji_dic,ji_alk,ji),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk),ji=1,2)
 WRITE(UNIT=jp_resunit) (((jndivg_atgen(ji_dic,ji_alk,ji),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk),ji=1,2)
 WRITE(UNIT=jp_resunit) (((jniter_atgen(ji_dic,ji_alk,ji),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk),ji=1,2)

 CLOSE(UNIT=jp_resunit)
#endif






 PRINT*
 PRINT*, 'Calling SOLVE_AT_GENERAL2_SEC'
 PRINT*, '-----------------------------'


#ifdef CASE_ZHINI
 z_hini = 1.E-8_wp
#endif

 jndivg_atsec(:,:, :) = 0

 CALL CPU_TIME(z_cputime_start)

DO ji_alk = 1, jp_nalk

  z_alktot = z_alktot_arr(ji_alk)

  DO ji_dic = 1, jp_ndic


#if CASE_DIC == 1
    z_dicvar = z_dicvar_arr(ji_dic)
#endif

#if CASE_DIC == 2
    z_dicvar = 10._wp**(z_dicvar_arr(ji_dic))
#endif

#if CASE_DIC == 3
    z_dicvar = z_dicvar_arr(ji_dic)
#endif

#if CASE_DIC == 4
    z_dicvar = 10._wp**(z_dicvar_arr(ji_dic))
#endif

    z_h = SOLVE_AT_GENERAL2_SEC(z_alktot, z_dicvar, z_bortot, &
                                          z_po4tot, z_siltot, &
                                          z_nh4tot, z_h2stot, &
                                          z_so4tot, z_flutot, &
                                          jp_dicsel, n_roots, &
#ifdef CASE_ZHINI
                                          p_hini=z_hini,      &
#endif
                                          p_val=z_val         )


    jnroots_atsec(ji_dic,ji_alk)          = n_roots
    jniter_atsec(ji_dic,ji_alk,:)         = niter_atsec(:)

    z_h_atsec(ji_dic,ji_alk, :)           = z_h(:)
    z_val_atsec(ji_dic,ji_alk, :)         = z_val(:)


    IF (n_roots > 0) THEN
      IF (z_h(1) == pp_hnan) &
      jndivg_atsec(ji_dic,ji_alk, 1)  = 1
    ENDIF

    IF (n_roots > 1) THEN
      IF (z_h(2) == pp_hnan) &
      jndivg_atsec(ji_dic,ji_alk, 2)  = 1
    ENDIF

  ENDDO

ENDDO


 CALL CPU_TIME(z_cputime_end)

 PRINT*
 PRINT*, '[DRIVER_AT_GENERAL2] elapsed time [s]           : ', z_cputime_end - z_cputime_start
 PRINT*, '[DRIVER_AT_GENERAL2] total number of calls      : ', jp_nalk*jp_ndic
 PRINT*, '[DRIVER_AT_GENERAL2] total number of divergences: ', SUM(jndivg_atsec)


#if defined(CREATEFILES)
 OPEN(UNIT=jp_resunit, FILE='atsec_'//cp_fileid//'.res',FORM='UNFORMATTED')

 WRITE(UNIT=jp_resunit) jp_ndic, jp_nalk
 WRITE(UNIT=jp_resunit) z_dicvar_min
 WRITE(UNIT=jp_resunit) z_dicvar_max
 WRITE(UNIT=jp_resunit) z_alktot_min
 WRITE(UNIT=jp_resunit) z_alktot_max
 WRITE(UNIT=jp_resunit) z_dicvar_arr
 WRITE(UNIT=jp_resunit) z_alktot_arr
 WRITE(UNIT=jp_resunit) z_bortot
 WRITE(UNIT=jp_resunit) z_po4tot
 WRITE(UNIT=jp_resunit) z_siltot
 WRITE(UNIT=jp_resunit) z_nh4tot
 WRITE(UNIT=jp_resunit) z_h2stot
 WRITE(UNIT=jp_resunit) z_so4tot
 WRITE(UNIT=jp_resunit) z_flutot

 WRITE(UNIT=jp_resunit) ((jnroots_atsec(ji_dic,ji_alk),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk)

 WRITE(UNIT=jp_resunit) (((z_h_atsec(ji_dic,ji_alk,ji),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk),ji=1,2)
 WRITE(UNIT=jp_resunit) (((z_val_atsec(ji_dic,ji_alk,ji),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk),ji=1,2)
 WRITE(UNIT=jp_resunit) (((jndivg_atsec(ji_dic,ji_alk,ji),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk),ji=1,2)
 WRITE(UNIT=jp_resunit) (((jniter_atsec(ji_dic,ji_alk,ji),ji_dic=1,jp_ndic),ji_alk=1,jp_nalk),ji=1,2)

 CLOSE(UNIT=jp_resunit)
#endif


!==============================================================================================
END PROGRAM
!==============================================================================================

